<?php

namespace Bolt\Extension\Ginger\Squarepay\Controller;

use Bolt\Controller\Base;
use Silex\Application;
use Silex\ControllerCollection;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;


/**
 * The controller for Square backend routes.
 * 
 * Get an OAuth Access Token, and store it - it's valid for 30 days,
 * then enters 15 day grace period during which it should usually be renewed
 *
 * @author Richard Knight <rich@apewave.com>
 */
class SquareBackendController extends Base
{
    public function __construct($config)
    {
        $this->config = $config;

        // Create and configure a new API client object
        $apiConfig = new \SquareConnect\Configuration();
        $apiConfig->setAccessToken($config['sq_app_secret']);
        $this->apiClient = new \SquareConnect\ApiClient($apiConfig);
    }    
    
    public function addRoutes(ControllerCollection $collection)
    {
        $collection->match('/', [$this, 'squareDashboard'])->before([$this, 'checkUserPermissions']);
        $collection->match('/request-oauth-token', [$this, 'oauthRequestToken'])->before([$this, 'checkUserPermissions']);
        $collection->match('/revoke-oauth-token', [$this, 'oauthRevokeToken'])->before([$this, 'checkUserPermissions']);
        return $collection;
    }

    public function squareDashboard()
    {
        $params = [
            'client_id' => $this->config['sq_app_id'],
            'scope' => 'ITEMS_READ MERCHANT_PROFILE_READ PAYMENTS_READ PAYMENTS_WRITE ORDERS_READ ORDERS_WRITE',
        ];
        $uri = 'https://'.$this->config['sq_domain'].$this->config['sq_auth_url'];
        $query = http_build_query($params, '', '&', PHP_QUERY_RFC3986);
        $auth_link = $uri.'?'.$query;
        $context = [
            'auth_link' => $auth_link, 
            'config' => $this->config
        ];
        return $this->render('@Squareback/dashboard.twig', $context);
    }

    public function oauthRequestToken(Application $app, Request $request)
    {
        try {
            $authorizationCode = $request->query->get('code');
            if (!$authorizationCode) {
                error_log('Authorization failed!');
                throw new \Exception('Error Processing Request: Authorization failed!', 1);
            }
            $oauthToken = $this->getOAuthToken($authorizationCode);
        } catch (Exception $e) {
            error_log($e->getMessage());
            return $this->render('@Squareback/error.twig', ['error' => $e->getMessage()]);
        }
        
        // Now store the token
        $manager = $app['filesystem'];
        if ($manager->hasFilesystem('extensions_config')) {
            $filesystem = $manager->getFilesystem('extensions_config');
            $filesystem->put('.sqoatoken', $oauthToken);
        } else {
            return $this->render('@Squareback/error.twig', ['error' =>'Could not write .squotoken to app/config/extensions/']);
        }
        return $this->render('@Squareback/authorised.twig', ['token' => $oauthToken]);
    }

    // -----------------------------------------------------------------------------------------------------------------

	public function checkUserPermissions()
	{
		$currentUser = $this->app['users']->getCurrentUser();
		$currentUserId = $currentUser['id'];
		if (!$this->app['users']->hasRole($currentUserId, 'root') && !$this->app['users']->hasRole($currentUserId, 'admin')) {
			return new RedirectResponse($this->generateUrl('dashboard'), Response::HTTP_SEE_OTHER);
		}
	}
	
	private function getOAuthToken($authorizationCode) {
        // Exchange the authorization code for an OAuth token
		
		// Create an OAuth API client
        $oauthApi = new \SquareConnect\Api\OAuthApi($this->apiClient);
        $body = new \SquareConnect\Model\ObtainTokenRequest();
      
        // Set the POST body
        $body->setClientId($this->config['sq_app_id']);
        $body->setClientSecret($this->config['sq_app_secret']);
        $body->setCode($authorizationCode);
        try {
            $result = $oauthApi->obtainToken($body);
        } catch (Exception $e) {
            error_log('Exception when calling OAuthApi->obtainToken: ' . $e->getMessage());
            throw new Exception('Error Processing Request: Token exchange failed!', 1);
        }
        return $result->getAccessToken();
	}
	
	public function oauthRevokeToken() {
		$manager = $this->app['filesystem'];
		if (!$manager->hasFilesystem('extensions_config')) {
			throw new Exception('Could not find filesystem extensions_config');
		}
		$filesystem = $manager->getFilesystem('extensions_config');
		$path = '.sqoatoken';
		$file = $filesystem->getFile($path);
		if (!$file->exists($path)) {
			throw new Exception('Could not find '.$path);
		}
		$oauthToken = $file->read($path);

		// Create an OAuth API client
		/*
		$oauthApi = new \SquareConnect\Api\OAuthApi($this->apiClient);
		$body = new \SquareConnect\Model\RevokeTokenRequest();

		// Set the POST body
		$body->setClientId($this->config['sq_app_id']);
		$body->setAccessToken($oauthToken);
		try {
			$result = $oauthApi->revokeToken($body);
			$filesytem->delete($path);
		} catch (Exception $e) {
			error_log('Exception when calling OAuthApi->revokeToken: ' . $e->getMessage());
			throw new Exception('Error Processing Request: Token revocation failed!', 1);
		}
		*/

		// Revoke the token - alternative method (as above didn't work)
		$url = 'https://connect.squareup.com/oauth2/revoke';
		$data = array(
			'access_token' => $oauthToken,
			'client_id' => $this->config['sq_app_id']
		);
		$options = array(
			'http' => array(
				'header'  => "Content-type: application/json\r\n".  
				"Authorization: Client ".$this->config['sq_app_secret']."\r\n",
				'method'  => 'POST',
				'content' => json_encode($data)
			)
		);
		$context = stream_context_create($options);
		$result = file_get_contents($url, false, $context);
		$filesytem->delete($path);
        return $this->render('@Squareback/revoked.twig');
	}

}
