<?php

namespace Bolt\Extension\Ginger\Squarepay;

use Bolt\Extension\Ginger\Squarepay\Api;
use Bolt\Extension\Ginger\Squarepay\Twig;
use Bolt\Extension\SimpleExtension;
use Bolt\Menu\MenuEntry;
use Silex\Application;

use Silex\ControllerCollection;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Squarepay extension class.
 *
 * @author Richard Knight <rich@apewave.com>
 */
class SquarepayExtension extends SimpleExtension
{
	private $apiClient = null;
	
	protected function registerServices(Application $app)
	{
		$app['square.apiClient'] = $app->share(function($app) {
			// Create and configure a new Square API client using the OAuth token
			$manager = $app['filesystem'];
			if (!$manager->hasFilesystem('extensions_config')) {
				throw new \Exception('Could not find filesystem extensions_config');
			}
			$filesystem = $manager->getFilesystem('extensions_config');
			$path = '.sqoatoken';
			$file = $filesystem->getFile($path);
			if (!$file->exists($path)) {
				throw new \Exception('Could not find '.$path);
			}
			$oauthToken = $file->read($path);

			$apiConfig = new \SquareConnect\Configuration();
			$apiConfig->setAccessToken($oauthToken);
			$apiClient = new \SquareConnect\ApiClient($apiConfig);
			return new \SquareConnect\ApiClient($apiConfig);
		});

		$config = $this->getConfig();
		if ($config['testmode']) {
			$app['square.apiSandboxClient'] = $app->share(function($app) use ($config) {
				// Create and configure a new Square API client using the OAuth token
				$apiConfig = new \SquareConnect\Configuration();
				$apiConfig->setAccessToken($config['sq_sandbox_token']);
				$apiClient = new \SquareConnect\ApiClient($apiConfig);
				return new \SquareConnect\ApiClient($apiConfig);
			});
		}

		$app['twig.runtime.square'] = $app->share(function ($app) {
			return new Twig\SquareTwigRuntime(
				$app['square.apiClient'],
				$app['twig'],
				$this->getConfig(),
				$app['session']->get('basket')
			);
		});

		$app['twig.runtimes'] = $app->extend(
			'twig.runtimes',
			function (array $runtimes) {
                // You must append your array to the passed in $runtimes array and return it
                return $runtimes + [
                    Twig\SquareTwigRuntime::class => 'twig.runtime.square',
                ];
			}
		);
	}

	protected function registerFrontendRoutes(ControllerCollection $collection)
	{
        $collection->post ('/basket/add',    [$this, 'basketAdd']);
        $collection->post ('/basket/remove', [$this, 'basketRemove']);
        $collection->post ('/basket/update', [$this, 'basketUpdate']);
        $collection->match('/basket/clear',  [$this, 'basketClear']);
		$collection->match('/process-square', [$this, 'processSquarePaymentResponse']);
	}
	
	protected function registerBackendControllers()
	{
		return [
			'/extensions/square' => new Controller\SquareBackendController($this->getConfig())
		];
	}

	protected function registerMenuEntries()
	{
		$menu = MenuEntry::create('square-menu', 'square')
			->setLabel('Square')
			->setIcon('fa:credit-card')
			->setPermission('settings')
			// ->setRoute('SquarepayExtension')
			;
		return [$menu];
	}

	protected function registerTwigPaths()
    {
		// Register Twig namespace to use in controllers
		return [
            'templates/backend'  => ['namespace' => 'Squareback'],
            'templates/frontend' => ['namespace' => 'Squarefront']
        ];
    }
	
	protected function registerTwigFunctions() {
        $getCatalog 	= [Twig\SquareTwigRuntime::class, 'getCatalog'];
        $getCatalogItem = [Twig\SquareTwigRuntime::class, 'getCatalogItem'];
        $getBasket 		= [Twig\SquareTwigRuntime::class, 'getBasket'];
        $cardform 		= [Twig\SquareTwigRuntime::class, 'displayCCForm'];
		return [
			'getcatalog' 	=> [ $getCatalog ],
			'getcatitem' 	=> [ $getCatalogItem ],
			'getbasket'		=> [ $getBasket, ['is_safe' => ['html']] ],
			'cardform' 		=> [ $cardform,  ['is_safe' => ['html']] ]
		];
	}

	// -----------------------------------------------------------------------------------------------------------------
	// BASKET

	public function basketAdd(Application $app, Request $request)
	{
        $basket	= $app['session']->get('basket');
		$sku	= $request->request->get('sku');
		$name	= $request->request->get('name');
		$price	= $request->request->get('price');
		$squid	= $request->request->get('squid'); // Square Unique Id (for CatalogItemVariation)
		$key	= substr(md5($sku), 0, 10);
		if (isset($basket[$key])) {
			$basket[$key]['quantity']++;
			$basket[$key]['total'] = $basket[$key]['quantity'] * $basket[$key]['price'];
		} else {
			$basket[$key] = [
				'sku'		=> $sku,
				'name'		=> $name,
				'price'		=> $price,
				'quantity'	=> 1,
				'total'		=> $price,
				'squid'		=> $squid
			];
		}
        $app['session']->set('basket', $basket);
        return new RedirectResponse('/basket');
	}

	public function basketUpdate(Application $app, Request $request)
	{
		$basket	= $app['session']->get('basket');
		$sku	= $request->request->get('sku');
		$action	= $request->request->get('action');
		$key	= substr(md5($sku), 0, 10);
		if (isset($basket[$key])) {
			if ($action === '+') {
				$basket[$key]['quantity']++;
			} else {
				$basket[$key]['quantity']--;
			}
			$basket[$key]['total'] = $basket[$key]['quantity'] * $basket[$key]['price'];
			if ($basket[$key]['quantity'] === 0) {
				unset($basket[$key]);
			} 
		}
        $app['session']->set('basket', $basket);
        return new RedirectResponse('/basket');
	}

	public function basketRemove(Application $app, Request $request)
	{
        $basket	= $app['session']->get('basket');
		$sku	= $request->request->get('sku');
		$key	= substr(md5($sku), 0, 10);
		unset($basket[$key]);
        $app['session']->set('basket', $basket);
        return new RedirectResponse('/basket');
	}

	public function basketClear(Application $app)
	{
        $app['session']->set('basket', []);
        return new RedirectResponse('/basket');
	}

	// -----------------------------------------------------------------------------------------------------------------
	// SQUARE PAYMENT PROCESSING

	public function processSquarePaymentResponse(Application $app, Request $request) {
		
		// Grab the card nonce, name and email
		$cardNonce = $request->request->get('nonce');
		$name = $request->request->get('name');
		$email = $request->request->get('email');
		$date = $request->request->get('date');
		$booker = $request->request->get('booker');
		
		$config = $this->getConfig();
		$testmode = $config['testmode'];
		$locationId = $testmode ? $config['sq_sandbox_location_id'] : $config['sq_location_id'];
		$apiClient = $testmode ? $app['square.apiSandboxClient'] : $app['square.apiClient'];

		// Create the order
		// NOTE: The Orders API has sandbox support for ad-hoc items only. There is no sandbox support for orders built from Catalog items.
		$orderId = null;
		$basket	= $app['session']->get('basket');
		$lineItems = [];
		foreach($basket as $line) {
			$sqitem = new \SquareConnect\Model\CreateOrderRequestLineItem;
			if ($config['testmode']) {
				$sqitem->setName($line['name']);
			} else {
				$sqitem->setCatalogObjectId($line['squid']);
			}
			$sqitem->setQuantity((string) $line['quantity']);
			$price = new \SquareConnect\Model\Money;
			$price->setAmount((int) $line['price']);
			$price->setCurrency('GBP');
			$sqitem->setBasePriceMoney($price);
			array_push($lineItems, $sqitem);
		}
		$request = new \SquareConnect\Model\CreateOrderRequest();
		$request->setIdempotencyKey(uniqid()); // uniqid() generates a random string
		$request->setLineItems($lineItems);
		try {
			$ordersApi = new \SquareConnect\Api\OrdersApi($apiClient);
			$response = $ordersApi->createOrder($locationId, $request);
			// dump($response);
		}
		catch (Exception $e) {
			// TODO: Add better error handling
			echo '<pre>';
			echo 'Exception when calling OrdersApi->createOrder:', $e->getMessage(), PHP_EOL;
			echo '</pre>';
			exit;
		}
		$orderId = $response['order']['id'];
		// dump($orderId);

		// Build transaction request
		$buyerInfo = [
			'buyer_email_address' => $email
		];
		$idempotencyKey = uniqid();
		$basketTotal = array_sum(array_column($basket, 'total'));
		$paymentInfo = [
			'idempotency_key' => $idempotencyKey,
			'amount_money' => [
				'amount' => $basketTotal,
				'currency' => 'GBP'
			],
			'card_nonce' => $cardNonce,
		];
		if ($orderId) {
			// Only in live, not in sandbox
			$paymentInfo['order_id'] = $orderId;
		}
		$referenceInfo = [
			'reference_id' => $name.' <'.$email.'>',
			'note' => $booker.' on '.$date
		];

		// $chargeRequest = new \SquareConnect\Model\ChargeRequest();
		// $chargeRequest->setOrderId($orderId); // Link an order ID

		$tranactionRequest = array_merge($buyerInfo, $paymentInfo, $referenceInfo);
		$transactionsApi = new \SquareConnect\Api\TransactionsApi($apiClient);
		try {
			$response = $transactionsApi->charge($locationId, $tranactionRequest);
		} catch (\SquareConnect\ApiException $e) {
			return new RedirectResponse('/order-problem');
		}

		$this->sendConfirmationEmail($app, $name, $email, $booker, $date, $basket, $basketTotal);
		$app['session']->set('basket', []);
        return new RedirectResponse('/order-complete');
	}

	private function sendConfirmationEmail($app, $name, $email, $booker, $date, $basket, $basketTotal)
	{
		$body = $app['twig']->render('@Squarefront/email-confirmation.twig', [
			'name'		=> $name,
			'booker'	=> $booker,
			'date'		=> $date,
			'basket'	=> $basket,
			'total'		=> $basketTotal
		]);

		$config = $this->getConfig();
		$bookingsEmail = $config['bookings_email'];
		$message = \Swift_Message::newInstance('The Grand Expedition Order Confirmation')
			->setFrom($bookingsEmail)
			->setTo($email, $name)
			->setBcc([$bookingsEmail, 'tge@apewave.com'])
			->setBody($body, 'text/html');
		if (filter_var($email, FILTER_VALIDATE_EMAIL)) {
			$message->setReplyTo($email, $name);
		}
		$sent = $app['mailer']->send($message);
		if (!$sent) {
			throw new \Exception("Message wasn't actually sent.");
		}
	}

}
